#include <crypt/crypt.h>
#include <util/time.h>

#include <stdio.h>
#include <stdint.h>
#include <stdlib.h>

#define MEGGA_BYTE     (1024*1024)

#define CNT       100000
#define KEY_SIZE      32
#define RED_SIZE     448
#define MSG_SIZE     428
#define SIG_SIZE       SHA384_SIZE
#define SHA_SIZE       SHA512_SIZE

static FILE *devrand;
static FILE *devnull;

static uint8_t key[KEY_SIZE];
static uint8_t red[RED_SIZE];
static uint8_t msg[MSG_SIZE];
static uint8_t sig[SIG_SIZE];
static uint8_t sha[SHA_SIZE];

static void snerpent_enc(void){
  uint8_t skey[SERPENT_SKEYSIZE];
  serpent_init(skey,key,KEY_SIZE);
  for(size_t x = 0;x < CNT;x++){
    serpent_cbc_enc(skey,red,red,RED_SIZE);
  }
}

static void snerpent_dec(void){
  uint8_t skey[SERPENT_SKEYSIZE];
  serpent_init(skey,key,KEY_SIZE);
  for(size_t x = 0;x < CNT;x++){
    serpent_cbc_dec(skey,red,red,RED_SIZE);
  }
}

static void hmac_sha_384(void){
  for(size_t x = 0;x < CNT;x++){
    hmac_sha384(sig,key,KEY_SIZE,red,RED_SIZE);
  }
}

static void sha_256(void){
  for(size_t x = 0;x < CNT;x++){
    sha256(sha,msg,MSG_SIZE);
  }
}

static void sha_512(void){
  for(size_t x = 0;x < CNT;x++){
    sha512(sha,msg,MSG_SIZE);
  }
}

#define MESURE(O,S) mesure(O,#O,S)
static void mesure(void (*func)(void),char const *op,size_t size){
  uint64_t s = mono_nsec();

  func();

  #define DEC 10
  uint64_t t      = mono_nsec()-s;
  uint64_t totel  = size*CNT;
  uint64_t ns     = t/CNT;
  uint64_t bw     = t == 0 ? -1 : totel*DEC*NSEC_SEC/MEGGA_BYTE/t;
  fprintf(stderr,"%-16s %6lu ns/call %5lu.%01lu MB/s\n",
          op,ns,bw/DEC,bw%DEC);
}

static FILE *foben(char const *path,char const *mode){
  FILE *f = fopen(path,mode);
  if(!f){
    perror("fopen");
    exit(1);
  }
  return f;
}

static void init(uint8_t *data,size_t size){
  if(fread(data,1,size,devrand) != size){
    perror("fread");
    exit(1);
  }
}

static void use(uint8_t *data,size_t size){
  fprintf(devnull,"use data so compilator dosnt get any good ideas %u %u %u\n",
          data[0],
          data[size/2],
          data[size-1]);
}

int main(void){
  devnull = foben("/dev/null",    "wb");
  devrand = foben("/dev/urandom", "rb");

  init(key,KEY_SIZE);
  init(red,RED_SIZE);
  init(msg,MSG_SIZE);
  init(sha,SHA_SIZE);
  init(sig,SIG_SIZE);

  MESURE(snerpent_enc,  RED_SIZE);
  MESURE(snerpent_dec,  RED_SIZE);
  MESURE(hmac_sha_384,  RED_SIZE);
  MESURE(sha_256,       MSG_SIZE);
  MESURE(sha_512,       MSG_SIZE);

  use(red,RED_SIZE);
  use(msg,MSG_SIZE);
  use(sha,SHA_SIZE);
  use(sig,SIG_SIZE);

  return 0;
}

