#ifndef SERIAL_H
#define SERIAL_H

#include <stdint.h>
#include <string.h>

#define host8le     (uint8_t)
#define host8be     (uint8_t)
#ifdef MACHINE_BIG_ENDIAN
  #define host16le  byteswap16
  #define host16be  (uint16_t)
  #define host32le  byteswap32
  #define host32be  (uint32_t)
  #define host64le  byteswap64
  #define host64be  (uint64_t)
#else
  #define host16le  (uint16_t)
  #define host16be  byteswap16
  #define host32le  (uint32_t)
  #define host32be  byteswap32
  #define host64le  (uint64_t)
  #define host64be  byteswap64
#endif

inline uint16_t byteswap16(uint16_t n);
inline uint32_t byteswap32(uint32_t n);
inline uint64_t byteswap64(uint64_t n);

inline uint16_t byteswap16(uint16_t n){
#ifdef __GNUC__
  return __builtin_bswap16(n);
#else
  return n>>8 | n<<8;
#endif
}

inline uint32_t byteswap32(uint32_t n){
#ifdef __GNUC__
  return __builtin_bswap32(n);
#else
  return (n              ) >> 24 |
         (n & 0x00FF0000u) >>  8 | 
         (n & 0x0000FF00u) <<  8 | 
         (n              ) << 24;
#endif
}

inline uint64_t byteswap64(uint64_t n){
#ifdef __GNUC__
  return __builtin_bswap64(n);
#else
  return (n                        ) >> 56 |
         (n & 0x00FF000000000000ull) >> 40 | 
         (n & 0x0000FF0000000000ull) >> 24 | 
         (n & 0x000000FF00000000ull) >>  8 | 
         (n & 0x00000000FF000000ull) <<  8 | 
         (n & 0x0000000000FF0000ull) << 24 | 
         (n & 0x000000000000FF00ull) << 40 | 
         (n                        ) << 56;
#endif
}

#endif

